
#include <stdint.h>
#include "type.h"
#include "string.c"

#define VA(x) (0x80000000 + (u32)x)

char *tab = "0123456789ABCDEF";
int BASE;
int color;

#include "kbd.c"
#include "vid.c"
#include "exceptions.c"
#include "queue.c"
#include "kernel.c"
#include "wait.c"
#include "fork.c"
#include "svc.c"
#include "sdc.c"

void copy_vectors(void) {
    extern uint32_t vectors_start;
    extern uint32_t vectors_end;
    uint32_t *vectors_src = &vectors_start;
    uint32_t *vectors_dst = (uint32_t *)0;
    while(vectors_src < &vectors_end)
       *vectors_dst++ = *vectors_src++;
}

int mkPtable()
{
  int i;
  u32 *ut = (u32 *)0x4000;   // at 16KB
  u32 entry = 0 | 0x412;     // 0x412;// AP=01 (Kmode R|W; Umode NO) domaian=0
  for (i=0; i<4096; i++)
    ut[i] = 0;

  for (i=0; i<512; i++){     // for 512 MB real memory 
    ut[i] = entry;
    entry += 0x100000;
  }
}

int kprintf(char *fmt, ...);

void IRQ_handler()
{
    int vicstatus, sicstatus;

    // read VIC status register to find out which interrupt
    vicstatus = VIC_STATUS;
    sicstatus = SIC_STATUS;  

    if (vicstatus & 0x80000000){
      if (sicstatus & (1<<3)){
          kbd_handler();
       }
       if (sicstatus & (1<<22)){
	   sdc_handler();
	 }
    }
}

int main()
{ 
   row = col = 0; 
   BASE = 10;
   color = RED;      
   fbuf_init();
   kprintf("                     Welcome to WANIX in Arm\n");
   kprintf("LCD display initialized : fbuf = %x\n", fb);
   color = WHITE;
   kbd_init();

   VIC_INTENABLE |= (1<<31);    // SIC to VIC's IRQ31
   
   /* enable KBD SDC IRQ */
   SIC_ENSET   |= (1<<3);   // KBD int=3 on SIC
   SIC_ENSET   |= (1<<22);  // SDC int=22 on SIC

   sdc_init();
   
   kernel_init();
   unlock();

   kfork("u1");    // P1 Umode image = /bin/u1 file
   
   color = CYAN;
   kprintf("P0 switch to P1 : ");
   kgetc();
   tswitch();  // switch to run P1 ==> never return again
}
